@@ -214,7 +214,7 @@ typedef struct grib_index codes_index;
  * @param err         :  0 if OK, integer value on error
  * @return            the newly created index
  */
-codes_index* codes_index_new_from_file(codes_context* c, char* filename, const char* keys, int* err);
+codes_index* codes_index_new_from_file(codes_context* c, const char* filename, const char* keys, int* err);
 
 /**
  *  Create a new index based on a set of keys.
@@ -320,7 +320,7 @@ int codes_index_select_double(codes_index* index, const char* key, double value)
  * @param value       : value of the key to select
  * @return            0 if OK, integer value on error
  */
-int codes_index_select_string(codes_index* index, const char* key, char* value);
+int codes_index_select_string(codes_index* index, const char* key, const char* value);
 
 /**
  *  Create a new handle from an index after having selected the key values.
@@ -1253,6 +1253,7 @@ int codes_points_get_values(codes_handle* h, codes_points* points, double* val);
 #define CODES_UTIL_PACKING_TYPE_GRID_SIMPLE_MATRIX  GRIB_UTIL_PACKING_TYPE_GRID_SIMPLE_MATRIX
 #define CODES_UTIL_PACKING_TYPE_GRID_SECOND_ORDER   GRIB_UTIL_PACKING_TYPE_GRID_SECOND_ORDER
 #define CODES_UTIL_PACKING_TYPE_CCSDS               GRIB_UTIL_PACKING_TYPE_CCSDS
+#define CODES_UTIL_PACKING_TYPE_IEEE                GRIB_UTIL_PACKING_TYPE_IEEE
 #define CODES_UTIL_PACKING_SAME_AS_INPUT            GRIB_UTIL_PACKING_SAME_AS_INPUT
 #define CODES_UTIL_PACKING_USE_PROVIDED             GRIB_UTIL_PACKING_USE_PROVIDED
 
