@@ -380,6 +380,8 @@ static int grib_load_smart_table(grib_context* c, const char* filename,
 
         numberOfColumns = 0;
         while (*s) {
+            char* tcol = t->entries[code].column[numberOfColumns];
+            if ( tcol ) grib_context_free_persistent(c, tcol);
             t->entries[code].column[numberOfColumns] = grib_context_strdup_persistent(c, s);
             numberOfColumns++;
             Assert(numberOfColumns < MAX_SMART_TABLE_COLUMNS);
@@ -569,6 +571,13 @@ static int unpack_long(grib_accessor* a, long* val, size_t* len)
     if (err)
         return 0;
 
+    if (*len < self->tableCodesSize) {
+        grib_context_log(a->context, GRIB_LOG_ERROR,
+                         " wrong size (%ld) for %s it contains %d values ", *len, a->name, self->tableCodesSize);
+        *len = 0;
+        return GRIB_ARRAY_TOO_SMALL;
+    }
+
     for (i = 0; i < self->tableCodesSize; i++)
         val[i] = self->tableCodes[i];
 
