@@ -8,11 +8,6 @@
  * virtue of its status as an intergovernmental organisation nor does it submit to any jurisdiction.
  */
 
-/*
- * C Implementation: grib_tools
- *
- */
-
 #include "grib_tools.h"
 #if HAVE_LIBJASPER
 /* Remove compiler warnings re macros being redefined */
@@ -150,6 +145,14 @@ int grib_tool(int argc, char** argv)
     grib_context* c        = grib_context_get_default();
     global_options.context = c;
 
+    /* This is a consequence of ECC-440.
+     * We want to keep the output file(s) opened as various
+     * messages are appended to them. Otherwise they will be opened/closed
+     * multiple times.
+     */
+    if (c->file_pool_max_opened_files == 0)
+        c->file_pool_max_opened_files = 200;
+
 #ifdef ENABLE_FLOATING_POINT_EXCEPTIONS
     feenableexcept(FE_ALL_EXCEPT & ~FE_INEXACT);
 #endif
@@ -177,8 +180,8 @@ int grib_tool(int argc, char** argv)
 
     /* ECC-926: Currently only GRIB indexing works. Disable the through_index if BUFR, GTS etc */
     if (global_options.mode == MODE_GRIB &&
-        is_grib_index_file(global_options.infile->name) &&
-        (global_options.infile_extra && is_grib_index_file(global_options.infile_extra->name))) {
+        is_index_file(global_options.infile->name) &&
+        (global_options.infile_extra && is_index_file(global_options.infile_extra->name))) {
         global_options.through_index = 1;
         return grib_tool_index(&global_options);
     }
@@ -358,7 +361,13 @@ static int grib_tool_without_orderby(grib_runtime_options* options)
                                       err != GRIB_SUCCESS)) {
             infile->handle_count++;
             options->handle_count++;
-            options->error = err;
+
+            if (c->no_fail_on_wrong_length && (err == GRIB_PREMATURE_END_OF_FILE || err == GRIB_WRONG_LENGTH))
+                err = 0;
+            if (!options->error) {
+                /* ECC-1086: Do not clear a previous error */
+                options->error = err;
+            }
 
             if (!h) {
                 /* fprintf(dump_file,"\t\t\"ERROR: unreadable message\"\n"); */
@@ -554,6 +563,9 @@ static int grib_tool_index(grib_runtime_options* options)
 
     navigate(options->index2->fields, options);
 
+    if (options->index2)
+        grib_context_free(c, options->index2->current);
+
     grib_tool_finalise_action(options);
 
     return 0;
@@ -935,8 +947,12 @@ static void get_value_for_key(grib_handle* h, const char* key_name, int key_type
     }
 
     if (ret != GRIB_SUCCESS) {
-        fprintf(dump_file, "Failed to get value for key %s\n", key_name);
-        exit(1);
+        if (ret == GRIB_NOT_FOUND) {
+            sprintf(value_str, "not_found");
+        } else {
+            fprintf(dump_file, "Failed to get value for key %s\n", key_name);
+            exit(1);
+        }
     }
 }
 
@@ -970,8 +986,10 @@ static int get_initial_element_of_array(grib_handle* h, const char* keyName, siz
             sval = (char*)grib_context_malloc(c, len * sizeof(char));
             if (!sval)
                 return GRIB_OUT_OF_MEMORY;
-            if ((err = grib_get_string(h, keyName, sval, &len)) != GRIB_SUCCESS)
+            if ((err = grib_get_string(h, keyName, sval, &len)) != GRIB_SUCCESS) {
+                free(sval);
                 return err;
+            }
             sprintf(value, "%s", sval);
             free(sval);
             break;
@@ -1054,6 +1072,9 @@ void grib_print_key_values(grib_runtime_options* options, grib_handle* h)
             /* ECC-236: Do not use grib_is_missing for BUFR */
             if (!grib_is_defined(h, options->print_keys[i].name))
                 ret = GRIB_NOT_FOUND;
+            if (ret == GRIB_SUCCESS) {
+                ret = grib_get_size(h, options->print_keys[i].name, &num_vals);
+            }
             if (ret == GRIB_SUCCESS) {
                 if (options->print_keys[i].type == GRIB_TYPE_UNDEFINED)
                     grib_get_native_type(h, options->print_keys[i].name, &(options->print_keys[i].type));
@@ -1065,18 +1086,22 @@ void grib_print_key_values(grib_runtime_options* options, grib_handle* h)
                             sprintf(value, "MISSING");
                         break;
                     case GRIB_TYPE_DOUBLE:
-                        ret = grib_get_double(h, options->print_keys[i].name, &dvalue);
-                        if (dvalue == GRIB_MISSING_DOUBLE)
-                            sprintf(value, "MISSING");
-                        else
-                            sprintf(value, options->format, dvalue);
+                        if (num_vals > 1) {
+                            ret = GRIB_ARRAY_TOO_SMALL;
+                        } else {
+                            ret = grib_get_double(h, options->print_keys[i].name, &dvalue);
+                            if (dvalue == GRIB_MISSING_DOUBLE) sprintf(value, "MISSING");
+                            else                               sprintf(value, options->format, dvalue);
+                        }
                         break;
                     case GRIB_TYPE_LONG:
-                        ret = grib_get_long(h, options->print_keys[i].name, &lvalue);
-                        if (lvalue == GRIB_MISSING_LONG)
-                            sprintf(value, "MISSING");
-                        else
-                            sprintf(value, "%ld", lvalue);
+                        if (num_vals > 1) {
+                            ret = GRIB_ARRAY_TOO_SMALL;
+                        } else {
+                            ret = grib_get_long(h, options->print_keys[i].name, &lvalue);
+                            if (lvalue == GRIB_MISSING_LONG) sprintf(value, "MISSING");
+                            else                             sprintf(value, "%ld", lvalue);
+                        }
                         break;
                     case GRIB_TYPE_BYTES:
                         ret = grib_get_string(h, options->print_keys[i].name, value, &len);
@@ -1156,7 +1181,7 @@ void grib_print_key_values(grib_runtime_options* options, grib_handle* h)
     if (options->latlon) {
         if (options->latlon_mode == 4) {
             int ii = 0;
-            for (ii = 0; ii < 4; ii++) {
+            for (ii = 0; ii < LATLON_SIZE; ii++) {
                 fprintf(dump_file, options->format, options->values[ii]);
                 fprintf(dump_file, " ");
             }
@@ -1259,9 +1284,10 @@ void grib_tools_write_message(grib_runtime_options* options, grib_handle* h)
     char filename[1024] = {0,};
     Assert(options->outfile != NULL && options->outfile->name != NULL);
 
-    if (options->error == GRIB_WRONG_LENGTH)
-        return;
-
+    /* See ECC-1086
+     * if (options->error == GRIB_WRONG_LENGTH)
+     *   return;
+     */
     if ((err = grib_get_message(h, &buffer, &size)) != GRIB_SUCCESS) {
         grib_context_log(h->context, GRIB_LOG_ERROR, "unable to get binary message\n");
         exit(err);
@@ -1333,10 +1359,10 @@ void grib_tools_write_message(grib_runtime_options* options, grib_handle* h)
     }
 #endif
 }
-int exit_if_input_is_directory(const char* tool_name, const char* filename)
+int exit_if_input_is_directory(const char* toolname, const char* filename)
 {
     if (path_is_directory(filename)) {
-        fprintf(stderr, "%s: ERROR: \"%s\": Is a directory\n", tool_name, filename);
+        fprintf(stderr, "%s: ERROR: \"%s\": Is a directory\n", toolname, filename);
         exit(1);
     }
     return 0;
